Setting up for local development
================================

Fog requires a PostgreSQL database to work, as well as to run the unit tests.

To install that locally on Ubuntu, do the following:
1. apt-get install postgresql postgresql-client postgresql-server-dev-all libpq-dev
2. Configure PostgreSQL to skip password authentication for local users - this is optional but simplifies local development.
    In order to do so, edit `/etc/postgresql/10/main/pg_hba.conf` and replace the following line:
        `host    all             all             127.0.0.1\/32            md5`
    with:
        `host    all             all             127.0.0.1\/32            trust`
   (See .circleci/config.yml for example on how this is done via a script)
3. Start the PostgreSQL server: `service postgresql start`
4. Create a PostgreSQL user that matches your current login username:
    `sudo -u postgres createuser --superuser $USER`
5. You should now be able to create a database: `createdb fog_test`
6. Install diesel-cli: `cargo install diesel_cli --no-default-features --features postgres`
7. To populate your newly created database with the fog tables, run this:
    `cd src/fog/sql_recovery_db && DATABASE_URL=postgres://$USER@localhost/fog_test diesel migration run`
8. Fog services that require connecting to the database need the DATABASE_URL environment variable set:
    `export DATABASE_URL=postgres://$USER@localhost/fog_test`
9. Running unit tests requires the TEST_DATABASE_URL environment variable:
    `export TEST_DATABASE_URL=postgres://localhost`
    Notice that it does not contain a database name - this gets automatically generated by the unit-test suite.


fog
===

FIXME: All this documentation is ancient and needs to be rewritten

Account-server related targets:

- `moblie_acct_ingest`: Ingests blocks from the main ledger, and writes the recovery database.
   grpc endpoints allow to
   - Get the pubkey associated to this ingest node
   - Provide a cert-chain signing the pubkey
   - Get the sealed private key associated to this ingest node
   - Reset the private key using a sealed value
   - Request the ingest node to connect to a consensus node, perform attestation,
     and provide the cert, with the goal of convincing it to publish (our) new
     pubkey to the blockchain, inducing the users to use it.
   - Accept and process a new block, decrypting the account hints and writing to the recovery database as appropriate

- `mobile_acct_view`: Support user requests by resolving against recovery database
   - For their seeds
   - For database rows corresponding to specific nonces
   - For their entire wallet (passing the `c` private key to the enclave temporarily)

- `mobile_acct_recovery_db`: Rust object corresponding to a database

- `mobile_acct_recovery_db_iface`: Trait used to abstract details of the database

- `mobile_acct_test_ingest_view` An integration test that creates a mock database,
   instantiates ingest and view nodes, creates a bunch of fake users and fake blocks,
   exercises ingest and view APIs and checks that users can find their transactions
   via the view api, even if they lose their wallet state, and even if the ingest
   node rotates keys.

- `mobile_acct_test_infra`: Mock database and mock user objects, these can be
  re-used across the integration test with real database and with mocked database
